/*
Vertical Pairing Diagram maker.
Copyright (C) 2010  J David Eisenberg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
class Grid {
  static final boolean SHOW_NUMBERS = true;
  static final boolean NO_NUMBERS = false;
  static final boolean TURN_ON = true;
  static final boolean TURN_OFF = false;
  static final int TITLE_HEIGHT = 12;
  static final int ROW_HEIGHT = 45;
  static final int DIVISIONS = 5;
  static final int DIV_HEIGHT = ROW_HEIGHT / DIVISIONS;
  static final int BOUT_OFFSET = 100;
  static final int BOUT_WIDTH = 50;
  static final int NUM_WIDTH = 25;
  static final int RESULT_WIDTH = 25;
  static final int NAME_OFFSET = 25;
  static final int POINT_OFFSET = 103;
  static final int CLASS_OFFSET = 137;
  static final int COMPETITOR_OFFSET = BOUT_OFFSET + RESULT_WIDTH;
  static final int PLACING_X = 450;
  static final int PLACING_WIDTH = 25;
  static final int POINT_WIDTH = 50;
  PFont titleFont;
  PFont smallFont;
  PFont medFont;
  PFont largeFont;
  String[] nameList;

  Grid(String[] nameList)
  {
    this.nameList = nameList;
    titleFont = createFont("Arial", 9);
    smallFont = createFont("Arial", 10);
    medFont = createFont("Arial", 11);
    largeFont = createFont("Arial", 18);

  }

  void display( )
  {
    String [] title = {
      "#", "Name", "1", "2", "3", "...",
      "RR1", "RR2", "RR3", "Place"               };
    int row = 0;
    int col = 0;
    int yPos = 0;
    rect(0, yPos, NUM_WIDTH, TITLE_HEIGHT);
    rect(NUM_WIDTH, yPos, 75, TITLE_HEIGHT);
    textFont(smallFont);
    textAlign(CENTER);
    for (int i = 0; i < 7; i++)
    {
      stroke(0);
      fill(255);
      rect(100 + i * BOUT_WIDTH, yPos, BOUT_WIDTH, TITLE_HEIGHT);
    }
    rect(450, yPos, PLACING_WIDTH, TITLE_HEIGHT);
    pushMatrix();
    translate(0, TITLE_HEIGHT);
    for (row = 0; row < 8; row++)
    {
      yPos = row * ROW_HEIGHT;
      rect(0, yPos, NUM_WIDTH, ROW_HEIGHT);
      rect(NUM_WIDTH, yPos, 75, ROW_HEIGHT);
      for(int i = 0; i < 7; i++)
      {
        rect(BOUT_OFFSET + i * BOUT_WIDTH, yPos, BOUT_WIDTH, ROW_HEIGHT);

        line(COMPETITOR_OFFSET + i * BOUT_WIDTH, yPos + DIV_HEIGHT,
        COMPETITOR_OFFSET + i * BOUT_WIDTH, yPos + ROW_HEIGHT);

        line(BOUT_OFFSET + i * BOUT_WIDTH, yPos + DIV_HEIGHT,
        BOUT_OFFSET + BOUT_WIDTH + i * BOUT_WIDTH, yPos + DIV_HEIGHT);

        for (int j = 2; j <= 4; j++)
        {
          line(BOUT_OFFSET + i * BOUT_WIDTH, yPos + j * DIV_HEIGHT,
          BOUT_OFFSET + RESULT_WIDTH + i * BOUT_WIDTH,
          yPos + j * DIV_HEIGHT);
        }
      }
      rect(450, yPos, PLACING_WIDTH, ROW_HEIGHT);
    }
    popMatrix();

    // draw headers
    noStroke();
    fill(0);
    textFont(titleFont);
    text(title[0], 12, TITLE_HEIGHT - 1);
    text(title[1], 62, TITLE_HEIGHT - 1);
    for (int i = 0; i < 7; i++)
    {
      text(title[i + 2], BOUT_OFFSET + RESULT_WIDTH + i * BOUT_WIDTH,
        TITLE_HEIGHT - 1);
    }
    text(title[9], 450 + PLACING_WIDTH / 2, TITLE_HEIGHT - 1);
  }

  void bigNumber(int n)
  {
    textFont(largeFont);
    textAlign(CENTER);
    pushStyle();
    fill(255);
    noStroke();
    rect(1, (n-1) * ROW_HEIGHT + 1,
      NUM_WIDTH - 1, ROW_HEIGHT - 1);
    popStyle();
    text(Integer.toString(n), NUM_WIDTH / 2,
    (n - 1) * ROW_HEIGHT +  (DIVISIONS - 1) * DIV_HEIGHT);
  }

  void showNames( )
  {
    pushMatrix();
    translate(0, TITLE_HEIGHT);
    for (int i = 0; i < 8; i++)
    {
      bigNumber(i + 1);
      textFont(medFont);
      textAlign(LEFT);
      text(nameList[i], NUM_WIDTH + 8,
      i * ROW_HEIGHT + (DIVISIONS - 1) * DIV_HEIGHT);
    }
    popMatrix();
  }


  int roundToColumn(int round)
  {
    int column = 0;
    if (round < 0)
    {
      column = 3 - round;
    }
    else
    {
      column = round - 1;
    }
    return column;
  }

  /*
   * Display the results for the bout in the given round and
   * row (1-based). The three period results are in p1, p2, and p3
   */
  void bout(int round, int row, String p1, String p2, String p3)
  {
    bout(round, row, p1, p2, p3, "");
  }

  /*
   * Display the results for the bout in the given round and
   * row (1-based). The three period results are in p1, p2, and p3.
   * Also display the classification points.
   */
  void bout(int round, int row, String p1, String p2, String p3,
  String classPoints)
  {
    int column = roundToColumn(round);
    int xPos;
    int yPos;
    row--;
    pushMatrix();
    translate(0, TITLE_HEIGHT);
    pushStyle();
    
    /* Clear out anything that used to be in the result area */
    pushStyle();
    fill(255);
    noStroke();
    xPos = BOUT_OFFSET + column * BOUT_WIDTH;
    for (int i = 2; i < 5; i++)
    {
      yPos = row * ROW_HEIGHT + i * DIV_HEIGHT;
      rect(xPos + 1, yPos + 1,
        RESULT_WIDTH - 1, DIV_HEIGHT - 1);
    }
    
    rect(xPos + RESULT_WIDTH + 1, row * ROW_HEIGHT 
      + DIV_HEIGHT + 1,
      NUM_WIDTH - 1, (DIVISIONS - 1) * DIV_HEIGHT - 1);
    
    popStyle();
    
    /* Display the results for each period */
    noStroke();
    textFont(smallFont);
    textAlign(LEFT);
    text(p1, POINT_OFFSET + column * BOUT_WIDTH,
    row * ROW_HEIGHT + 3 * DIV_HEIGHT - 1);
    text(p2, POINT_OFFSET + column * BOUT_WIDTH,
    row * ROW_HEIGHT + 4 * DIV_HEIGHT - 1);
    text(p3, POINT_OFFSET + column * BOUT_WIDTH,
    row * ROW_HEIGHT + 5 * DIV_HEIGHT - 1);
    popStyle();
    
    /* "X" should be drawn as lines, not the letter X */
    if (classPoints.equals("X"))
    {
      xPos = BOUT_OFFSET  + RESULT_WIDTH + column * BOUT_WIDTH;
      yPos = row * ROW_HEIGHT + DIV_HEIGHT;
      line(xPos, yPos, xPos + RESULT_WIDTH,
        yPos + (DIVISIONS - 1) * DIV_HEIGHT);
      line(xPos, yPos + (DIVISIONS - 1) * DIV_HEIGHT,
        xPos + RESULT_WIDTH, yPos);
    }
    else
    {
      pushStyle();
      noStroke();
      textFont(largeFont);
      textAlign(CENTER);
      text(classPoints, CLASS_OFFSET + column * BOUT_WIDTH,
      (row * ROW_HEIGHT + (DIVISIONS - 1) * DIV_HEIGHT) - 1);
      popStyle();
    }
    popMatrix();
  }

  /*
   Set up numbers for person n1 against n2;
   if n2 or n1 is 0, that means it's a "bye"
   
   If bigNumbers is true, show large numbers in leftmost column of bracket
   */
  void pair(int round, int n1, int n2, boolean bigNumbers)
  {
    int column = roundToColumn(round);
    int row1 = n1 - 1;
    int row2 = n2 - 1;
    int xPos = BOUT_OFFSET + column * BOUT_WIDTH + RESULT_WIDTH / 2;
    String str1 = (n1 == 0) ? "Bye" : Integer.toString(n1);
    String str2 = (n2 == 0) ? "Bye" : Integer.toString(n2);
    /* Look at n1's row */
    pushMatrix();
    translate(0, TITLE_HEIGHT);
    pushStyle();
    noStroke();
    textAlign(CENTER);
    textFont(smallFont);
    if (n1 > 0)
    {
      text(str2, xPos, row1 * ROW_HEIGHT + 2 * DIV_HEIGHT - 1);
    }

    if (n2 > 0)
    {
      text(str1, xPos, row2 * ROW_HEIGHT + 2 * DIV_HEIGHT - 1);
    }

    if (bigNumbers)
    {
      if (n1 > 0)
      {
        bigNumber(n1);
      }
      if (n2 > 0)
      {
        bigNumber(n2);
      }
    }

    popStyle();
    popMatrix();
  }

  void pair(int round, int n1, int n2)
  {
    pair(round, n1, n2, false);
  }

  void highlightPair(int round, int n1, int n2, boolean turnOn)
  {
    highlightPair(round, n1, n2, turnOn, NO_NUMBERS);
  }
  
  void highlightPair(int round, int n1, int n2, boolean turnOn, boolean showNumbers)
  {
    int column = roundToColumn(round);
    pushStyle();
    pushMatrix();
    translate(0, TITLE_HEIGHT);
    if (turnOn)
    {
      fill(255, 255, 128);
    }
    else
    {
      fill(255);
    }
    noStroke();
    clearPair(column, n1, n2);
    popMatrix();
    popStyle();
    pair(round, n1, n2, showNumbers);
  }

  void erasePair(int round, int n1, int n2)
  {
    int column = roundToColumn(round);
    pushStyle();
    pushMatrix();
    translate(0, TITLE_HEIGHT);
    fill(255);
    noStroke();
    clearPair(column, n1, n2);
    popMatrix();
    popStyle();
  }

  void clearPair(int column, int n1, int n2)
  {
    if (n1 > 0)
    {
      rect(BOUT_OFFSET + column * BOUT_WIDTH + 1,
      (n1 - 1) * ROW_HEIGHT + DIV_HEIGHT + 1, RESULT_WIDTH - 1,
      DIV_HEIGHT - 1);
    }
    if (n2 > 0)
    {
      rect(BOUT_OFFSET + column * BOUT_WIDTH + 1,
      (n2 - 1) * ROW_HEIGHT + DIV_HEIGHT + 1, RESULT_WIDTH - 1,
      DIV_HEIGHT - 1);
    }
  }

  void eliminate(int round, int n)
  {
    pushStyle();
    pushMatrix();
    translate(0, TITLE_HEIGHT);
    stroke(255, 128, 128);
    strokeWeight(2);
    noFill();
    line(0, n * ROW_HEIGHT, NUM_WIDTH, (n-1) * ROW_HEIGHT);
    line(PLACING_X, n * ROW_HEIGHT, PLACING_X + PLACING_WIDTH,
    (n-1) * ROW_HEIGHT);
    line(BOUT_OFFSET + round * BOUT_WIDTH + 2,
      (n-1)*ROW_HEIGHT,
      BOUT_OFFSET + round * BOUT_WIDTH + 2,
      n * ROW_HEIGHT);
    popMatrix();
    popStyle();
  }
  
  void drawRRArc(int n)
  {
    pushMatrix();
    translate(0, TITLE_HEIGHT);
    pushStyle();
    fill(255, 128, 128);
    noStroke();
    arc(BOUT_OFFSET + 4 * BOUT_WIDTH,
      (n-1)*ROW_HEIGHT + ROW_HEIGHT / 2,
      BOUT_WIDTH / 3, ROW_HEIGHT, PI/2, 3 * PI/ 2);
    popStyle();
    popMatrix();
  }
  
  void arrow(int round, int n)
  {
    int column = roundToColumn(round);
    int xPos;
    float yPos;
    pushMatrix();
    translate(0, TITLE_HEIGHT);
    pushStyle();
    xPos = BOUT_OFFSET + column * BOUT_WIDTH;
    yPos = (n - 1) * ROW_HEIGHT + 3.5 * DIV_HEIGHT;
    line(xPos + 4, yPos, xPos + RESULT_WIDTH - 4, yPos);
    line(xPos + RESULT_WIDTH - 4, yPos, xPos + RESULT_WIDTH - 7,
      yPos - 3);
    line(xPos + RESULT_WIDTH - 4, yPos, xPos + RESULT_WIDTH - 7,
      yPos + 3);
    popStyle();
    popMatrix();
  }
  
  void showPlace(int competitor, int place)
  {
    pushMatrix();
    translate(0, TITLE_HEIGHT);
    textFont(largeFont);
    textAlign(CENTER);
    pushStyle();
    fill(255);
    noStroke();
    rect(PLACING_X +1, (competitor-1) * ROW_HEIGHT + 1,
      NUM_WIDTH - 1, ROW_HEIGHT - 1);
    popStyle();
    text(Integer.toString(place), PLACING_X + PLACING_WIDTH / 2,
    (competitor - 1) * ROW_HEIGHT +  (DIVISIONS - 1) * DIV_HEIGHT);
    popMatrix();
  }
}



