/*
Vertical Pairing Diagram maker.
Copyright (C) 2010  J David Eisenberg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

String[] nameList = {
  "Alan", "Bob", "Cuong", "Donald", "Enrique",
  "Frank", "Greg", "Hiro" };
final color BLUE = color(0, 0, 255);

Grid grid;
void setup( )
{
  size(Grid.NUM_WIDTH +
    Grid.PLACING_WIDTH + 8 * Grid.BOUT_WIDTH + Grid.PLACING_WIDTH + 1,
    Grid.TITLE_HEIGHT + 8 * Grid.ROW_HEIGHT + 1);
  background(255);
  smooth();
  grid = new Grid(nameList);
  grid.display();
  save("empty_grid.png");
  grid.showNames();
  save("name_num.png");
  
  pair1();
  pair2();
 
  enter1();
  enter2();
  pair3();

  enter3();
  roundrobin();
  
}

void pair1()
{
  // round 1 , stage 1
  fill(BLUE);
  grid.pair(1, 2, 1, Grid.SHOW_NUMBERS);
  save("rnd1.png");
  
  // round 1, pair 3 & 4
  fill(0);
  grid.pair(1, 2, 1, Grid.SHOW_NUMBERS);
  fill(BLUE);
  grid.pair(1, 3, 4, Grid.SHOW_NUMBERS);
  save("rnd1b.png");
  
  // round 1, pair everyone
  fill(0);
  grid.pair(1, 3, 4, Grid.SHOW_NUMBERS);
  grid.pair(1, 5, 6);
  grid.pair(1, 7, 8);
  save("rnd1all.png");
}

void pair2()
{
  // gray out round 1; start pairing round 2
  fill(128);
  grid.pair(1, 1, 2);
  grid.pair(1, 3, 4);
  grid.pair(1, 5, 6);
  grid.pair(1, 7, 8);
  fill(BLUE);
  grid.highlightPair(2, 1, 3, Grid.TURN_ON, Grid.SHOW_NUMBERS);
  save("rnd2.png");

  // unhighlight 1 & 3, pair 2 & 4
  fill(0);
  grid.highlightPair(2, 1, 3, Grid.TURN_OFF, Grid.SHOW_NUMBERS);
  fill(BLUE);
  grid.highlightPair(2, 2, 4, Grid.TURN_ON, Grid.SHOW_NUMBERS);
  save("rnd2b.png");
  
  fill(0);
  grid.highlightPair(2, 2, 4, Grid.TURN_OFF, Grid.SHOW_NUMBERS);
  grid.pair(2, 5, 7);
  grid.pair(2, 6, 8);
  grid.pair(1, 1, 2); // and restore round 1 to black
  grid.pair(1, 3, 4);
  grid.pair(1, 5, 6);
  grid.pair(1, 7, 8);
  save("rnd2all.png");
}

void enter1()
{
  // reset the bout numbers, and enter first match score.
  fill(0);
  grid.highlightPair(1, 1, 2, Grid.TURN_OFF);
  grid.highlightPair(1, 3, 4, Grid.TURN_OFF);
  grid.highlightPair(1, 5, 6, Grid.TURN_OFF);
  grid.highlightPair(1, 7, 8, Grid.TURN_OFF);
  grid.pair(1, 1, 2);
  grid.pair(1, 3, 4);
  grid.pair(1, 5, 6);
  grid.pair(1, 7, 8);
  fill(BLUE);
  stroke(BLUE);
  grid.bout(1, 1, "7-3", "4-1", "", "3");
  grid.bout(1, 2, "3-7", "1-4", "", "X");
  save("score1.png");
  
  // reset first match; add a fall
  fill(0);
  stroke(0);
  grid.bout(1, 1, "7-3", "4-1", "", "3");
  grid.bout(1, 2, "3-7", "1-4", "", "X");
  fill(BLUE);
  stroke(BLUE);
  grid.bout(1, 3, "LF","","","X");
  grid.bout(1, 4, "1:57", "", "", "5");
  save("score1b.png");
 
  // reset second match; add a tech fall
  fill(0);
  stroke(0);
  grid.bout(1, 3, "LF","","","X");
  grid.bout(1, 4, "1:57", "", "", "5");
  fill(BLUE);
  stroke(BLUE);
  grid.bout(1, 5, "6-0", "9-2", "", "4");
  grid.bout(1, 6, "0-6", "2-9", "", "X");
  save("score1c.png");

  // reset third match; add a regular decision
  fill(0);
  stroke(0);
  grid.bout(1, 5, "6-0", "9-2", "", "4");
  grid.bout(1, 6, "0-6", "2-9", "", "X");
  fill(BLUE);
  stroke(BLUE);
  grid.bout(1, 7, "0-3", "5-3", "6-1", "3");
  grid.bout(1, 8, "3-0", "3-5", "1-6", "X");
  save("score1all.png");
}
void enter2()
{
   // reset last match; add a TF in round 2
  fill(0);
  stroke(0);
  grid.bout(1, 7, "0-3", "5-3", "6-1", "3");
  grid.bout(1, 8, "3-0", "3-5", "1-6", "X");
  fill(BLUE);
  stroke(BLUE);
  grid.bout(2, 1, "7-0", "9-2", "", "4");
  grid.bout(2, 3, "0-7", "2-9", "", "X");
  save("score2.png");
  
  // Person 3, round 2
  fill(0);
  stroke(0);
  grid.bout(2, 1, "7-0", "9-2", "", "4");
  grid.bout(2, 3, "0-7", "2-9", "", "X");
  
  grid.eliminate(2, 3);
  save("score2b.png");
  
  fill(BLUE);
  stroke(BLUE);
  grid.bout(2, 2, "1:26", "", "", "5");
  grid.bout(2, 4, "LF", "", "", "X");
  save("score2c.png");
  
  fill(0);
  stroke(0);
  grid.bout(2, 2, "1:26", "", "", "5");
  grid.bout(2, 4, "LF", "", "", "X");
  grid.bout(2, 5, "3-0", "1-7", "2-9", "X");
  grid.bout(2, 7, "0-3", "7-1", "9-2", "4");
  grid.bout(2, 6, "2-5", "1-5", "", "X");
  grid.bout(2, 8, "5-2", "5-1", "", "3");
  grid.eliminate(2, 6);
  save("score2all.png");
} 

void pair3()
{
  fill(BLUE);
  stroke(BLUE);
  grid.highlightPair(3, 1, 4, Grid.TURN_ON);
  save("rnd3a.png");
  grid.highlightPair(3, 1, 4, Grid.TURN_OFF);
  grid.highlightPair(3, 2, 5, Grid.TURN_ON);
  fill(0);
  stroke(0);
  grid.pair(3, 1, 4, Grid.SHOW_NUMBERS);
  save("rnd3b.png");
  grid.highlightPair(3, 2, 5, Grid.TURN_OFF);
  fill(255, 0, 0);
  stroke(255, 0, 0);
  grid.highlightPair(1, 7, 8, Grid.TURN_ON);
  grid.highlightPair(3, 7, 8, Grid.TURN_ON);
  save("rnd3c.png");
  
  // undo this match and previous match
  grid.highlightPair(1, 7, 8, Grid.TURN_OFF);
  grid.erasePair(3, 7, 8);
  grid.highlightPair(3, 2, 5, Grid.TURN_ON);
  fill(0);
  stroke(0);
  grid.pair(1, 7, 8);
  save("rnd3c2.png");
  
  //back force 7 vs. 2
  grid.highlightPair(3, 2, 5, Grid.TURN_OFF);
  fill(BLUE);
  stroke(BLUE);
  grid.highlightPair(3, 2, 7, Grid.TURN_ON);
//  grid.pair(3, 2, 7);
  save("rnd3d.png");
  
  // pair 5 & 8 to complete round 3
  grid.highlightPair(3, 2, 7, Grid.TURN_OFF);
  grid.highlightPair(3, 5, 8, Grid.TURN_ON);
//  grid.pair(3, 5, 8);
  fill(0);
  stroke(0);
  grid.pair(3, 2, 7);
  save("rnd3e.png");
}

void enter3()
{
  // turn off highlighting of 5 & 8
  fill(0);
  stroke(0);
  grid.highlightPair(3, 5, 8, Grid.TURN_OFF);
  grid.pair(3, 5, 8);
  save("score3.png");
  
  grid.bout(3, 1, "1:12", "", "", "5");
  grid.bout(3, 4, "LF", "", "", "X");
  grid.bout(3, 7, "6-1", "6-2", "", "3");
  grid.bout(3, 2, "1-6", "2-6", "", "X");
  grid.bout(3, 5, "0-3", "4-0", "5-1", "3");
  grid.bout(3, 8, "3-0", "0-4", "1-5", "X");
  grid.eliminate(3, 2);
  grid.eliminate(3, 4);
  grid.eliminate(3, 8);
  save("score3a.png");
}

void roundrobin()
{
  grid.drawRRArc(1);
  grid.drawRRArc(5);
  grid.drawRRArc(7);
  save("rr.png");
  
  grid.pair(-1, 1, 5);
  grid.pair(-1, 7, 0);
  grid.pair(-2, 7, 1);
  grid.pair(-2, 5, 0);
  grid.pair(-3, 1, 0);
  grid.pair(-3, 5, 7);
  grid.bout(-1, 7, "", " \u2014", "", "0");
  grid.bout(-2, 5, "", " \u2014", "", "0");
  grid.bout(-3, 1, "", " \u2014", "", "0");
  save("rr2.png");
  
  fill(BLUE);
  stroke(BLUE);
  grid.highlightPair(2, 5, 7, Grid.TURN_ON);
  grid.highlightPair(-3, 5, 7, Grid.TURN_ON);
// grid.pair(2, 5, 7);
//  grid.pair(-3, 5, 7);
  save("rr3.png");
  
  grid.highlightPair(-3, 5, 7, Grid.TURN_ON);
//  grid.pair(-3, 5, 7);
  grid.bout(-3, 5, "", "", "", "1");
  grid.bout(-3, 7, "", "", "", "4");
  grid.arrow(-3, 5);
  grid.arrow(-3, 7);
  fill(0);
  stroke(0);
  grid.highlightPair(2, 5, 7, Grid.TURN_OFF);
//  grid.pair(2, 5, 7);
  save("rr4.png");
  
  // restore coloration to RR 3
  grid.highlightPair(-3, 5,7, Grid.TURN_OFF);
//  grid.pair(-3, 5, 7);
  grid.bout(-3, 5, "", "", "", "1");
  grid.bout(-3, 7, "", "", "", "4");
  grid.arrow(-3, 5);
  grid.arrow(-3, 7);

  // enter last 2 bouts
  grid.bout(-1, 5, "0-3", "1:18", "", "5");
  grid.bout(-1, 1, "3-0", "LF", "", "0");
  grid.bout(-2, 7, "4-2", "4-1", "", "3");
  grid.bout(-2, 1, "2-4", "1-4", "", "1");
  save("rr5.png");
  
  // placing
  fill(BLUE);
  noStroke();
  grid.showPlace(7, 1);
  grid.showPlace(5, 2);
  grid.showPlace(1, 3);
  save("place.png");
}
